
"""
# =========== Use these import statements for conversion functions ============

from engcoolprop.conv_funcs import (Aeng_fromSI, CondEng_fromSI, CPeng_fromSI, 
                                    Deng_fromSI, Peng_fromSI, rhoEng_fromRhoSI, Seng_fromSI, 
                                    Teng_fromSI, UHeng_fromSI, Veng_fromSI)
from engcoolprop.conv_funcs import (ASI_fromEng, CondSI_fromEng, CPSI_fromEng, 
                                    DSI_fromEng, PSI_fromEng, rhoSI_fromRhoEng, SSI_fromEng, 
                                    TSI_fromEng, UHSI_fromEng, VSI_fromEng)
from engcoolprop.conv_funcs import  EchoInput


"""
def Aeng_fromSI( A ):
    """Return ft/sec from m/sec"""
    return A * 3.28084
    
def ASI_fromEng( A ):
    """Return m/sec from ft/sec"""
    return A / 3.28084

def CondEng_fromSI( Cond ):
    """Return BTU/ft-hr-R from W/m/K"""
    return Cond * 0.578176

def CondSI_fromEng( Cond ):
    """Return W/m/K from BTU/ft-hr-R"""
    return Cond / 0.578176

def CPeng_fromSI( S ):
    """Return BTU/lbm degR from J/kg/K"""
    return S * 0.0002388461111111111

def CPSI_fromEng( S ):
    """Return J/kg/K from BTU/lbm degR"""
    return S / 0.0002388461111111111

def Deng_fromSI( D ): # 1 lbm/ft^3 = 16.01843417 kg/m^3
    """Return lbm/cu ft from kg/m^3"""
    return D  / 16.01843417

def DSI_fromEng( D ):
    """Return kg/m^3 from lbm/cu ft"""
    return D * 16.01843417

def EchoInput( x ):
    """Return input unchanged (can be float, string, int, etc.)"""
    return x

def Peng_fromSI( P ):
    """Return psia from Pa"""
    return P / 6894.76

def PSI_fromEng( P ):
    """Return Pa from psia"""
    return P * 6894.76

def rhoEng_fromRhoSI( rho ):
    """Return lbm/cuin from  g/cm^3"""
    return rho / 27.679905

def rhoSI_fromRhoEng( rho ):
    """Return g/cm^3 from lbm/cuin"""
    return rho * 27.679905
    
def Seng_fromSI( S ):
    """Return BTU/lbm degR from J/kg/K"""
    return S * 0.0002388461111111111

def SSI_fromEng( S ):
    """Return J/kg/K from BTU/lbm degR"""
    return S / 0.0002388461111111111

def Teng_fromSI( T ):
    """Return degR from degK"""
    return T * 1.8

def TSI_fromEng( T ):
    """Return degK from degR"""
    return T / 1.8

def UHeng_fromSI( UH ): # 1 BTU/lbm = 2326 J/kg
    """Return BTU/lbm from J/kg"""
    return UH / 2326.0

def UHSI_fromEng( UH ):
    """Return J/kg from BTU/lbm"""
    return UH * 2326.0

def Veng_fromSI( V ):
    """Return lbm/ft/sec from Pa-s"""
    # AS.viscosity() returns Pa*s
    return V * 0.6719689751 # convert from Pa*s to lbm/ft/sec
    
def VSI_fromEng( V ):
    """Return Pa-s from lbm/ft/sec"""
    return V / 0.6719689751




if 0:
    from engcoolprop.ec_incomp_fluid import EC_Incomp_Fluid

    # Create incompressible object. (without specifying state point)
    ec_inc = EC_Incomp_Fluid(symbol="Water", auto_fix_value_errors=True, show_warnings=2 )

    # Print state point
    ec_inc.printProps() # Print state point at given T,P

    print()
    ec_inc.printTPD()
    print()
    ec_inc.printTransport()

else:
    from engcoolprop.ec_incomp_fluid import EC_Incomp_Fluid

    # Create incompressible object at T=500 degR, P=500 psia and max pressure = 5000 psia
    ec_inc = EC_Incomp_Fluid(symbol="Water", T=500, P=500, Pmax=5000)

    # Print state point
    ec_inc.printProps()

    #ec_inc.setTP( 600, 600 )
    #ec_inc.printProps()

print( '='*66 )

# printSIUnits
from engcoolprop.ec_incomp_fluid import EC_Incomp_Fluid

# Create incompressible object. (without specifying state point)
ec_inc = EC_Incomp_Fluid(symbol="Water", T=500, P=500, Pmax=5000,
                         auto_fix_value_errors=True, show_warnings=0 )

# Print state point with SI units
ec_inc.printSIProps() # Print state point at given T,P



import matplotlib.pyplot as plt
import numpy as np
from engcoolprop.ec_incomp_fluid import EC_Incomp_Fluid

# Create incompressible solution object. (without specifying state point)
ec_inc = EC_Incomp_Fluid(symbol="DowQ", auto_fix_value_errors=True, show_warnings=0 ) # T=degR, P=psia

tArr = np.linspace(500, 800, 50)

# Use a few different pressures
for P in [10000, 5000, 2000, 15]:

    densL = []
    for T in tArr:
        ec_inc.setTP( T, P)
        densL.append( ec_inc.H )

    plt.plot( tArr, densL, label="P = %g"%P)

plt.grid( True )
plt.title( 'DowQ Enthalpy')
plt.xlabel( 'Temperature (degR)')
plt.ylabel( 'Enthalpy (BTU/lbm)')
plt.legend( loc='best' )

plt.savefig( 'DowQ_enthalpy.png', dpi=200)
plt.show()




from engcoolprop.conv_funcs import (ASI_fromEng, CondSI_fromEng, CPSI_fromEng, 
                                    DSI_fromEng, PSI_fromEng, rhoSI_fromRhoEng, SSI_fromEng, 
                                    TSI_fromEng, UHSI_fromEng, VSI_fromEng)
from engcoolprop.conv_funcs import  EchoInput
from engcoolprop.parameter_units import si_unitsD
    
# map SI from Engineering conversion functions for each of the fluid properties
toSI_callD = {} # index=eng prop name, value=conversion func (e.g. SSI_fromEng)

class SI_obj:
    def __init__(self, eng_obj, propL):
        """
        Create an object of SI units from one of Engineering units
        eng_obj = the object having properties in Engineering units
        propL = names of properties to be converted to SI and made properties of SI_obj
        """
        self.propL = propL
        self.max_g_len = 8 # find max %g string length

        self.si_valueD = {} # key:prop name, value:SI value

        for prop in propL:
            # get Engineering value from eng_obj
            val = getattr(eng_obj, prop)

            
            # Convert Engineering value into SI value
            if callable(val):# might be a method name, i.e. callable, so check for that
                si_val = toSI_callD[prop](val())
            else:                
                si_val = toSI_callD[prop](val)
            self.si_valueD[prop] = si_val

            # find longest %g string representing SI value
            try:
                si_val_str = '%g'%si_val
                self.max_g_len = max(self.max_g_len, len(si_val_str) )
            except:
                pass

        # create formats for g and s 
        self.g_fmt = '%' + '%ig'%self.max_g_len


    def __getattr__(self, name):
        """
        Override __getattr__ to return right justified string of the maximum length.

        Parameters:
        name (str): The name of the attribute to access.

        Returns:
        str: The right justified string of the maximum length.
        """
        if name in self.si_valueD:
            try:
                s = self.g_fmt%self.si_valueD[name]
            except:
                s = self.si_valueD[name].rjust(self.max_g_len)
            return s
        else:
            return '?'*self.max_g_len


    def show_si_values(self):
        for prop in self.propL:
            print( '%20s'%prop, getattr(self, prop), si_unitsD.get(prop, ''))

toSI_callD['Cond']     = CondSI_fromEng
toSI_callD['Condmax']  = CondSI_fromEng
toSI_callD['Condmin']  = CondSI_fromEng
toSI_callD['Cp']       = CPSI_fromEng
toSI_callD['Cpmax']    = CPSI_fromEng
toSI_callD['Cpmin']    = CPSI_fromEng
toSI_callD['Cv']       = CPSI_fromEng
toSI_callD['D']        = DSI_fromEng
toSI_callD['Dc']       = DSI_fromEng
toSI_callD['Dmax']     = DSI_fromEng
toSI_callD['Dmin']     = DSI_fromEng
toSI_callD['E']        = UHSI_fromEng
toSI_callD['Emax']     = UHSI_fromEng
toSI_callD['Emin']     = UHSI_fromEng
toSI_callD['fluid']    = EchoInput
toSI_callD['gamma']    = EchoInput # NOTE: gamma is a function
toSI_callD['good_nbp'] = EchoInput
toSI_callD['H']        = UHSI_fromEng
toSI_callD['Hmax']     = UHSI_fromEng
toSI_callD['Hmin']     = UHSI_fromEng
toSI_callD['name']     = EchoInput
toSI_callD['P']        = PSI_fromEng
toSI_callD['Pc']       = PSI_fromEng
toSI_callD['percentage']    = EchoInput
toSI_callD['pcent_max_str'] = EchoInput
toSI_callD['pcent_min_str'] = EchoInput
toSI_callD['Pinput']   = PSI_fromEng
toSI_callD['Pmax']     = PSI_fromEng
toSI_callD['Pmin']     = PSI_fromEng
toSI_callD['psat_is_supported'] = EchoInput
toSI_callD['Psat']     = PSI_fromEng
toSI_callD['Psat_max'] = PSI_fromEng
toSI_callD['Psat_min'] = PSI_fromEng
toSI_callD['Q']        = EchoInput
toSI_callD['rho']      = rhoSI_fromRhoEng # NOTE: this is g/cm^3 from kg/m^3
toSI_callD['rho_max']  = rhoSI_fromRhoEng # NOTE: this is g/cm^3 from kg/m^3
toSI_callD['rho_min']  = rhoSI_fromRhoEng # NOTE: this is g/cm^3 from kg/m^3
toSI_callD['S']        = SSI_fromEng
toSI_callD['Smax']     = SSI_fromEng
toSI_callD['Smin']     = SSI_fromEng
toSI_callD['sonicV']   = ASI_fromEng
toSI_callD['symbol']   = EchoInput
toSI_callD['T']        = TSI_fromEng
toSI_callD['T_freeze'] = TSI_fromEng
toSI_callD['Tc']       = TSI_fromEng
toSI_callD['Tmax']     = TSI_fromEng
toSI_callD['Tmin']     = TSI_fromEng
toSI_callD['Tnbp']     = TSI_fromEng
toSI_callD['Ttriple']  = TSI_fromEng
toSI_callD['Visc']     = VSI_fromEng
toSI_callD['Viscmax']  = VSI_fromEng
toSI_callD['Viscmin']  = VSI_fromEng
toSI_callD['WtMol']    = EchoInput
toSI_callD['Z']        = EchoInput


if __name__ == "__main__":

    from engcoolprop.ec_fluid import EC_Fluid
    eng_obj = EC_Fluid( symbol='H2O' )
    eng_obj.printProps()

    ec_fluidL = ['Cond', 'Cp', 'Cv', 'D', 'Dc', 'E', 'gamma', 'good_nbp', 'H', 'name', 
         'P', 'Pc', 'Q', 'S', 'sonicV', 'symbol', 'T', 'Tc', 'Tnbp', 'Ttriple', 
         'Visc', 'WtMol', 'Z']

    si_obj = SI_obj( eng_obj, ec_fluidL)
    si_obj.show_si_values()




input_set = set() # set of input params

output_set = set() # set of output params

param_si_unitsD = {} # key:name, value:SI units
param_eng_unitsD= {} # key:name, value:Engineering units

param_descD = {} # key:name, value:description
param_synonymD = {} # key:name, value: set of all synonyms

preferred_nameD = {} # key:preferred name, value:description


# ============= input_set ===================
input_set = {'DewPoint', 'Sha', 'T_dp', 'H', 'P', 'W', 'psi_w', 'R', 'Tdb', 'Entropy', 'RelHum', 
             'WetBulb', 'Enthalpy', 'T_db', 'Hda', 'D', 'T', 'V', 'B', 'Y', 'Vha', 'Omega', 'Sda', 
             'S', 'Hha', 'HumRat', 'Twb', 'P_w', 'RH', 'Vda', 'T_wb', 'Tdp'}

input_list = sorted( input_set, key=str.lower)

# ============= output_set ===================
output_set = {'DewPoint', 'Visc', 'mu', 'Sha', 'Z', 'T_dp', 'M', 'cv_ha', 'H', 'P', 'W', 'psi_w', 
              'CV', 'R', 'Tdb', 'Entropy', 'RelHum', 'CVha', 'WetBulb', 'Enthalpy', 'k', 'T_db', 
              'Hda', 'D', 'cp', 'T', 'V', 'B', 'Y', 'Vha', 'Omega', 'Sda', 'S', 'Hha', 'HumRat', 
              'Twb', 'Conductivity', 'P_w', 'RH', 'Cha', 'C', 'Vda', 'T_wb', 'cp_ha', 'Tdp', 'K'}

output_list = sorted( output_set, key=str.lower)

# ============= param_si_unitsD ===================
param_si_unitsD["B"] = "K"
param_si_unitsD["Twb"] = "K"
param_si_unitsD["T_wb"] = "K"
param_si_unitsD["WetBulb"] = "K"
param_si_unitsD["C"] = "J/kg dry air/K"
param_si_unitsD["cp"] = "J/kg dry air/K"
param_si_unitsD["Cha"] = "J/kg humid air/K"
param_si_unitsD["cp_ha"] = "J/kg humid air/K"
param_si_unitsD["CV"] = "J/kg dry air/K"
param_si_unitsD["CVha"] = "J/kg humid air/K"
param_si_unitsD["cv_ha"] = "J/kg humid air/K"
param_si_unitsD["D"] = "K"
param_si_unitsD["Tdp"] = "K"
param_si_unitsD["DewPoint"] = "K"
param_si_unitsD["T_dp"] = "K"
param_si_unitsD["H"] = "J/kg dry air"
param_si_unitsD["Hda"] = "J/kg dry air"
param_si_unitsD["Enthalpy"] = "J/kg dry air"
param_si_unitsD["Hha"] = "J/kg humid air"
param_si_unitsD["K"] = "W/m/K"
param_si_unitsD["k"] = "W/m/K"
param_si_unitsD["Conductivity"] = "W/m/K"
param_si_unitsD["M"] = "Pa-s"
param_si_unitsD["Visc"] = "Pa-s"
param_si_unitsD["mu"] = "Pa-s"
param_si_unitsD["psi_w"] = "mol water/mol humid air"
param_si_unitsD["Y"] = "mol water/mol humid air"
param_si_unitsD["P"] = "Pa"
param_si_unitsD["P_w"] = "Pa"
param_si_unitsD["R"] = ""
param_si_unitsD["RH"] = ""
param_si_unitsD["RelHum"] = ""
param_si_unitsD["S"] = "J/kg dry air/K"
param_si_unitsD["Sda"] = "J/kg dry air/K"
param_si_unitsD["Entropy"] = "J/kg dry air/K"
param_si_unitsD["Sha"] = "J/kg humid air/K"
param_si_unitsD["T"] = "K"
param_si_unitsD["Tdb"] = "K"
param_si_unitsD["T_db"] = "K"
param_si_unitsD["V"] = "m^3 /kg dry air"
param_si_unitsD["Vda"] = "m^3 /kg dry air"
param_si_unitsD["Vha"] = "m^3 /kg humid air"
param_si_unitsD["W"] = "kg water/kg dry air"
param_si_unitsD["Omega"] = "kg water/kg dry air"
param_si_unitsD["HumRat"] = "kg water/kg dry air"
param_si_unitsD["Z"] = ""

# Non-CoolProp names
param_si_unitsD["MolWt"] = "g/gmole"
param_si_unitsD["Cond"] = "W/m/K"

# ============= param_eng_unitsD ===================
param_eng_unitsD["B"] = "degR"
param_eng_unitsD["Twb"] = "degR"
param_eng_unitsD["T_wb"] = "degR"
param_eng_unitsD["WetBulb"] = "degR"
param_eng_unitsD["C"] = "BTU/lbm dry air/degR"  
param_eng_unitsD["cp"] = "BTU/lbm dry air/degR"
param_eng_unitsD["Cha"] = "BTU/lbm humid air/degR"
param_eng_unitsD["cp_ha"] = "BTU/lbm humid air/degR"
param_eng_unitsD["CV"] = "BTU/lbm dry air/degR"
param_eng_unitsD["CVha"] = "BTU/lbm humid air/degR"
param_eng_unitsD["cv_ha"] = "BTU/lbm humid air/degR"
param_eng_unitsD["D"] = "degR"
param_eng_unitsD["Tdp"] = "degR"
param_eng_unitsD["DewPoint"] = "degR"
param_eng_unitsD["T_dp"] = "degR"
param_eng_unitsD["H"] = "BTU/lbm dry air"  
param_eng_unitsD["Hda"] = "BTU/lbm dry air"
param_eng_unitsD["Enthalpy"] = "BTU/lbm dry air"
param_eng_unitsD["Hha"] = "BTU/lbm humid air"
param_eng_unitsD["K"] = "BTU/ft-hr-R"
param_eng_unitsD["k"] = "BTU/ft-hr-R"
param_eng_unitsD["Conductivity"] = "BTU/ft-hr-R"
param_eng_unitsD["M"] = "[1.0E5 * lbm/ft-sec]"
param_eng_unitsD["Visc"] = "[1.0E5 * lbm/ft-sec]"
param_eng_unitsD["mu"] = "[1.0E5 * lbm/ft-sec]"
param_eng_unitsD["psi_w"] = "mol water/mol humid air"
param_eng_unitsD["Y"] = "mol water/mol humid air"
param_eng_unitsD["P"] = "psia"
param_eng_unitsD["P_w"] = "psia"
param_eng_unitsD["R"] = ""
param_eng_unitsD["RH"] = ""
param_eng_unitsD["RelHum"] = ""
param_eng_unitsD["S"] = "BTU/lbm dry air/degR"
param_eng_unitsD["Sda"] = "BTU/lbm dry air/degR"
param_eng_unitsD["Entropy"] = "BTU/lbm dry air/degR"
param_eng_unitsD["Sha"] = "BTU/lbm humid air/degR"
param_eng_unitsD["T"] = "degR"
param_eng_unitsD["Tdb"] = "degR"
param_eng_unitsD["T_db"] = "degR"
param_eng_unitsD["V"] = "ft^3/lbm dry air"
param_eng_unitsD["Vda"] = "ft^3/lbm dry air"
param_eng_unitsD["Vha"] = "ft^3/lbm humid air"
param_eng_unitsD["W"] = "lbm water/lbm dry air"
param_eng_unitsD["Omega"] = "lbm water/lbm dry air"
param_eng_unitsD["HumRat"] = "lbm water/lbm dry air"
param_eng_unitsD["Z"] = ""

# Non-CoolProp names
param_eng_unitsD["MolWt"] = "lbm/lbmmole"
param_eng_unitsD["Cond"] = "BTU/ft-hr-R"


# ============= param_descD ===================
param_descD["B"] = "Wet-Bulb Temperature"
param_descD["Twb"] = "Wet-Bulb Temperature"
param_descD["T_wb"] = "Wet-Bulb Temperature"
param_descD["WetBulb"] = "Wet-Bulb Temperature"
param_descD["C"] = "Mixture Cp per unit dry air"
param_descD["cp"] = "Mixture Cp per unit dry air"
param_descD["Cha"] = "Mixture Cp per unit humid air"
param_descD["cp_ha"] = "Mixture Cp per unit humid air"
param_descD["CV"] = "Mixture Cv per unit dry air"
param_descD["CVha"] = "Mixture Cv per unit humid air"
param_descD["cv_ha"] = "Mixture Cv per unit humid air"
param_descD["D"] = "Dew-Point Temperature"
param_descD["Tdp"] = "Dew-Point Temperature"
param_descD["DewPoint"] = "Dew-Point Temperature"
param_descD["T_dp"] = "Dew-Point Temperature"
param_descD["H"] = "Mixture enthalpy per dry air"
param_descD["Hda"] = "Mixture enthalpy per dry air"
param_descD["Enthalpy"] = "Mixture enthalpy per dry air"
param_descD["Hha"] = "Mixture enthalpy per humid air"
param_descD["K"] = "Mixture thermal conductivity"
param_descD["k"] = "Mixture thermal conductivity"
param_descD["Conductivity"] = "Mixture thermal conductivity"
param_descD["M"] = "Mixture viscosity"
param_descD["Visc"] = "Mixture viscosity"
param_descD["mu"] = "Mixture viscosity"
param_descD["psi_w"] = "Water mole fraction"
param_descD["Y"] = "Water mole fraction"
param_descD["P"] = "Pressure"
param_descD["P_w"] = "Partial pressure of water vapor"
param_descD["R"] = "Relative humidity in range [0, 1]"
param_descD["RH"] = "Relative humidity in range [0, 1]"
param_descD["RelHum"] = "Relative humidity in range [0, 1]"
param_descD["S"] = "Mixture entropy per unit dry air"
param_descD["Sda"] = "Mixture entropy per unit dry air"
param_descD["Entropy"] = "Mixture entropy per unit dry air"
param_descD["Sha"] = "Mixture entropy per unit humid air"
param_descD["T"] = "Dry-Bulb Temperature"
param_descD["Tdb"] = "Dry-Bulb Temperature"
param_descD["T_db"] = "Dry-Bulb Temperature"
param_descD["V"] = "Mixture volume per unit dry air"
param_descD["Vda"] = "Mixture volume per unit dry air"
param_descD["Vha"] = "Mixture volume per unit humid air"
param_descD["W"] = "Humidity Ratio"
param_descD["Omega"] = "Humidity Ratio"
param_descD["HumRat"] = "Humidity Ratio"
param_descD["Z"] = "Compressibility factor (Z=pv/(RT))"

# Non-CoolProp names
param_descD["MolWt"] = "Mixture Molecular Weight"
param_descD["Cond"] = "Mixture thermal conductivity"

# ============= param_synonymD ===================
param_synonymD["B"] = "{'Twb', 'B', 'T_wb', 'WetBulb'}"
param_synonymD["Twb"] = "{'Twb', 'B', 'T_wb', 'WetBulb'}"
param_synonymD["T_wb"] = "{'Twb', 'B', 'T_wb', 'WetBulb'}"
param_synonymD["WetBulb"] = "{'Twb', 'B', 'T_wb', 'WetBulb'}"
param_synonymD["C"] = "{'C', 'cp'}"
param_synonymD["cp"] = "{'C', 'cp'}"
param_synonymD["Cha"] = "{'cp_ha', 'Cha'}"
param_synonymD["cp_ha"] = "{'cp_ha', 'Cha'}"
param_synonymD["CV"] = "{'CV'}"
param_synonymD["CVha"] = "{'CVha', 'cv_ha'}"
param_synonymD["cv_ha"] = "{'CVha', 'cv_ha'}"
param_synonymD["D"] = "{'Tdp', 'T_dp', 'D', 'DewPoint'}"
param_synonymD["Tdp"] = "{'Tdp', 'T_dp', 'D', 'DewPoint'}"
param_synonymD["DewPoint"] = "{'Tdp', 'T_dp', 'D', 'DewPoint'}"
param_synonymD["T_dp"] = "{'Tdp', 'T_dp', 'D', 'DewPoint'}"
param_synonymD["H"] = "{'Hda', 'Enthalpy', 'H'}"
param_synonymD["Hda"] = "{'Hda', 'Enthalpy', 'H'}"
param_synonymD["Enthalpy"] = "{'Hda', 'Enthalpy', 'H'}"
param_synonymD["Hha"] = "{'Hha'}"
param_synonymD["K"] = "{'Conductivity', 'k', 'K'}"
param_synonymD["k"] = "{'Conductivity', 'k', 'K'}"
param_synonymD["Conductivity"] = "{'Conductivity', 'k', 'K'}"
param_synonymD["M"] = "{'mu', 'M', 'Visc'}"
param_synonymD["Visc"] = "{'mu', 'M', 'Visc'}"
param_synonymD["mu"] = "{'mu', 'M', 'Visc'}"
param_synonymD["psi_w"] = "{'Y', 'psi_w'}"
param_synonymD["Y"] = "{'Y', 'psi_w'}"
param_synonymD["P"] = "{'P'}"
param_synonymD["P_w"] = "{'P_w'}"
param_synonymD["R"] = "{'RH', 'R', 'RelHum'}"
param_synonymD["RH"] = "{'RH', 'R', 'RelHum'}"
param_synonymD["RelHum"] = "{'RH', 'R', 'RelHum'}"
param_synonymD["S"] = "{'Sda', 'Entropy', 'S'}"
param_synonymD["Sda"] = "{'Sda', 'Entropy', 'S'}"
param_synonymD["Entropy"] = "{'Sda', 'Entropy', 'S'}"
param_synonymD["Sha"] = "{'Sha'}"
param_synonymD["T"] = "{'T_db', 'Tdb', 'T'}"
param_synonymD["Tdb"] = "{'T_db', 'Tdb', 'T'}"
param_synonymD["T_db"] = "{'T_db', 'Tdb', 'T'}"
param_synonymD["V"] = "{'Vda', 'V'}"
param_synonymD["Vda"] = "{'Vda', 'V'}"
param_synonymD["Vha"] = "{'Vha'}"
param_synonymD["W"] = "{'W', 'Omega', 'HumRat'}"
param_synonymD["Omega"] = "{'W', 'Omega', 'HumRat'}"
param_synonymD["HumRat"] = "{'W', 'Omega', 'HumRat'}"
param_synonymD["Z"] = "{'Z'}"

param_synonymD["Cond"] = "{'Conductivity', 'k', 'K'}"

# convert param_synonymD values from strings to sets
for k,v in param_synonymD.items():
    param_synonymD[k] = eval(v)

# ============= preferred_nameD ===================
preferred_nameD["WetBulb"] = "Wet-Bulb Temperature"
preferred_nameD["cp"]    = "Mixture Cp per unit dry air"
preferred_nameD["cp_ha"] = "Mixture Cp per unit humid air"
preferred_nameD["CV"]    = "Mixture Cv per unit dry air"
preferred_nameD["CVha"]  = "Mixture Cv per unit humid air"
preferred_nameD["DewPoint"] = "Dew-Point Temperature"
preferred_nameD["Hda"] = "Mixture enthalpy per dry air"
preferred_nameD["Hha"] = "Mixture enthalpy per humid air"
preferred_nameD["Conductivity"] = "Mixture thermal conductivity"
preferred_nameD["Visc"] = "Mixture viscosity"
preferred_nameD["Y"] = "Water mole fraction"
preferred_nameD["P"] = "Pressure"
preferred_nameD["P_w"] = "Partial pressure of water vapor"
preferred_nameD["RelHum"] = "Relative humidity in range [0, 1]"
preferred_nameD["Sda"] = "Mixture entropy per unit dry air"
preferred_nameD["Sha"] = "Mixture entropy per unit humid air"
preferred_nameD["Tdb"] = "Dry-Bulb Temperature"
preferred_nameD["Vda"] = "Mixture volume per unit dry air"
preferred_nameD["Vha"] = "Mixture volume per unit humid air"
preferred_nameD["HumRat"] = "Humidity Ratio"
preferred_nameD["Z"] = "Compressibility factor (Z=pv/(RT))"

preferred_list = sorted( [k for k in preferred_nameD.keys()], key=str.lower)
preferred_set  = set( preferred_list )

if __name__ == "__main__":

    # make sure that all the descriptions are covered by preferred names
    desc_set = set()
    for v in param_descD.values():
        desc_set.add( v )

    pref_desc_set = set()
    for v in preferred_nameD.values():
        pref_desc_set.add( v )

    print( 'len(desc_set)      =', len(desc_set))
    print( 'len(pref_desc_set) =', len(pref_desc_set))





import unittest
# import unittest2 as unittest # for versions of python < 2.7

"""
        Method                            Checks that
self.assertEqual(a, b)                      a == b   
self.assertNotEqual(a, b)                   a != b   
self.assertTrue(x)                          bool(x) is True  
self.assertFalse(x)                         bool(x) is False     
self.assertIs(a, b)                         a is b
self.assertIsNot(a, b)                      a is not b
self.assertIsNone(x)                        x is None 
self.assertIsNotNone(x)                     x is not None 
self.assertIn(a, b)                         a in b
self.assertNotIn(a, b)                      a not in b
self.assertIsInstance(a, b)                 isinstance(a, b)  
self.assertNotIsInstance(a, b)              not isinstance(a, b)  
self.assertAlmostEqual(a, b, places=5)      a within 5 decimal places of b
self.assertNotAlmostEqual(a, b, delta=0.1)  a is not within 0.1 of b
self.assertGreater(a, b)                    a is > b
self.assertGreaterEqual(a, b)               a is >= b
self.assertLess(a, b)                       a is < b
self.assertLessEqual(a, b)                  a is <= b

for expected exceptions, use:

with self.assertRaises(Exception):
    blah...blah...blah

with self.assertRaises(KeyError):
    blah...blah...blah

Test if __name__ == "__main__":


    def test__main__(self):
        # Change bottom of source file to call "dev_tests"
        
         def dev_tests():
            pass

         if __name__ == "__main__":
            dev_tests()
            
        # then test by calling <name>.dev_tests()


See:
      https://docs.python.org/2/library/unittest.html
         or
      https://docs.python.org/dev/library/unittest.html
for more assert options
"""

import sys, os

from engcoolprop.ec_humid_air import EC_Humid_Air, dev_tests
from engcoolprop.humid_air_params import *

here = os.path.abspath(os.path.dirname(__file__)) # Needed for py.test
up_one = os.path.split( here )[0]  # Needed to find models development version
if here not in sys.path[:2]:
    sys.path.insert(0, here)
if up_one not in sys.path[:2]:
    sys.path.insert(0, up_one)

class MyTest(unittest.TestCase):


    def test_should_always_pass_cleanly(self):
        """Should always pass cleanly."""
        pass

    def test_myclass_existence(self):
        """Check that myclass exists"""
        ec_ha = EC_Humid_Air()
        
        # See if the self.EC_Incomp_Fluid object exists
        self.assertIsInstance(ec_ha, EC_Humid_Air, msg=None)

    def test_print_IO(self):
        """test print IO"""

        ec_ha = EC_Humid_Air()
        ec_ha.print_input_params()
        ec_ha.print_output_params()
        ec_ha.printProps()
        ec_ha.printProps( eng_units=False )


    def test_bad_input_pair(self):
        """test bad_input_pair"""
        
        ec_ha = EC_Humid_Air( Tdb=530.0, RelHum=0.5 )

        ec_ha2 = EC_Humid_Air( HumRat=ec_ha.HumRat, RelHum=ec_ha.RelHum )
        self.assertEqual(  ec_ha2.Cond, float('inf') )
        

    def test_degF(self):
        """test degF"""
        ec_ha = EC_Humid_Air( Tdb=530.0, RelHum=0.5 )

        ec_ha2 = EC_Humid_Air(  TdbdegF=70.33, RelHum=0.5  )
        
        self.assertAlmostEqual(ec_ha2.Cond, ec_ha.Cond, places=5)

    def test_wrong_number_of_inputs(self):
        """test wrong number of inputs"""

        with self.assertRaises(ValueError):
            ec_ha = EC_Humid_Air( Tdb=530.0 )


    def test_omitted_pressure(self):
        """test omitted pressure"""
        
        with self.assertRaises(ValueError):
            ec_ha = EC_Humid_Air( Tdb=530.0, WetBulb=520, DewPoint=510 )

    def test_two_water_contents(self):
        """test two water contents"""
        
        ec_ha = EC_Humid_Air( Tdb=530.0, RelHum=0.5 )

        ec_ha2 = EC_Humid_Air(  RelHum=ec_ha.RelHum, DewPoint=ec_ha.DewPoint  )
        
        with self.assertRaises(AssertionError):
            self.assertAlmostEqual(ec_ha2.Cond, ec_ha.Cond, places=5)

    def test__main__(self):
        old_sys_argv = list(sys.argv)
        sys.argv = list(sys.argv)
        sys.argv.append('suppress_show')
        
        try:
            dev_tests()
        except:
            raise Exception('ERROR... failed in __main__ routine')
        finally:
            sys.argv = old_sys_argv
        

if __name__ == '__main__':
    # Can test just this file from command prompt
    #  or it can be part of test discovery from nose, unittest, pytest, etc.
    unittest.main()




""" 
Humid Air Properties
http://www.coolprop.org/fluid_properties/HumidAir.html#table-of-inputs-outputs-to-hapropssi

"""
import traceback
import CoolProp.CoolProp as CP
from engcoolprop.humid_air_params import (preferred_list, param_synonymD, param_eng_unitsD, 
                                          param_descD, param_si_unitsD, preferred_nameD, 
                                          input_set)

from engcoolprop.conv_funcs import (CPeng_fromSI ,  CondEng_fromSI ,   DSI_fromEng ,  CPSI_fromEng,
                                  Deng_fromSI ,   PSI_fromEng, Peng_fromSI, EchoInput,
                                  SSI_fromEng ,  Seng_fromSI ,  TSI_fromEng ,  
                                  Teng_fromSI ,  UHSI_fromEng ,  UHeng_fromSI ,   Veng_fromSI,
                                  CondSI_fromEng, VSI_fromEng  )
from engcoolprop.banner import banner

def Voleng_fromSI( Vol ): # 1 lbm/ft^3 = 16.01843417 kg/m^3
    return Vol * 16.01843417

def VolSI_fromEng( Vol ): # 1 lbm/ft^3 = 16.01843417 kg/m^3
    return Vol / 16.01843417

PSIA_PER_ATM = 14.6959 # used to convert between psia and atm

DEG_F_R_OFFSET = 459.67


print_orderL = ["Tdb","WetBulb","DewPoint","P","P_w","",
"Vda","Vha","","cp","cp_ha","CV","CVha","","Hda","Hha","Sda","Sha",
"",     "Cond","Visc","","RelHum","HumRat","Y","Z"]


# map Eng to SI conversion functions for each of the fluid properties
toSI_callD = {} # index=eng, value=conversion func (e.g. SSI_fromEng)
toSI_callD['Conductivity'] = CondSI_fromEng
toSI_callD['cp'] = CPSI_fromEng
toSI_callD['cp_ha'] = CPSI_fromEng
toSI_callD['CV'] = CPSI_fromEng
toSI_callD['CVha'] = CPSI_fromEng
toSI_callD['DewPoint'] = TSI_fromEng
toSI_callD['Hda'] = UHSI_fromEng
toSI_callD['Hha'] = UHSI_fromEng
toSI_callD['HumRat'] = EchoInput
toSI_callD['P'] = PSI_fromEng
toSI_callD['P_w'] = PSI_fromEng
toSI_callD['RelHum'] = EchoInput
toSI_callD['Sda'] = SSI_fromEng
toSI_callD['Sha'] = SSI_fromEng
toSI_callD['Tdb'] = TSI_fromEng
toSI_callD['Vda'] = VolSI_fromEng
toSI_callD['Vha'] = VolSI_fromEng
toSI_callD['Visc'] = VSI_fromEng
toSI_callD['WetBulb'] = TSI_fromEng
toSI_callD['Y'] = EchoInput
toSI_callD['Z'] = EchoInput

toSI_callD['Cond'] = CondSI_fromEng

# need to add to toSI_callD with all synonyms of preferred names
for pref_name in preferred_list:
    syn_set = param_synonymD[pref_name]
    for syn in syn_set:
        if syn not in toSI_callD:
            toSI_callD[syn] = toSI_callD[pref_name]

# print( 'toSI_callD.keys =', sorted(toSI_callD.keys(), key=str.lower))
# ==========================================================
# map SI to Eng conversion functions for each of the fluid properties
toEng_callD = {} # index=si char, value=conversion func (e.g. Seng_fromSI)

toEng_callD['Conductivity'] = CondEng_fromSI
toEng_callD['cp'] = CPeng_fromSI
toEng_callD['cp_ha'] = CPeng_fromSI
toEng_callD['CV'] = CPeng_fromSI
toEng_callD['CVha'] = CPeng_fromSI
toEng_callD['DewPoint'] = Teng_fromSI
toEng_callD['Hda'] = UHeng_fromSI
toEng_callD['Hha'] = UHeng_fromSI
toEng_callD['HumRat'] = EchoInput
toEng_callD['P'] = Peng_fromSI
toEng_callD['P_w'] = Peng_fromSI
toEng_callD['RelHum'] = EchoInput
toEng_callD['Sda'] = Seng_fromSI
toEng_callD['Sha'] = Seng_fromSI
toEng_callD['Tdb'] = Teng_fromSI
toEng_callD['Vda'] = Voleng_fromSI
toEng_callD['Vha'] = Voleng_fromSI
toEng_callD['Visc'] = Veng_fromSI
toEng_callD['WetBulb'] = Teng_fromSI
toEng_callD['Y'] = EchoInput
toEng_callD['Z'] = EchoInput

toEng_callD['Cond'] = CondEng_fromSI

# may need to show viscosity in lbf-s/ft^2 ???
def convert_viscosity_to_lbf_s_per_ft2(lbm_per_s_per_ft):
    # Conversion factor
    conversion_factor = 0.03108095
    # Convert to lbf-s/ft^2
    lbf_s_per_ft2 = lbm_per_s_per_ft * conversion_factor
    return lbf_s_per_ft2

    # Example usage
    # viscosity_lbm_per_s_per_ft = 1.23324e-5
    # viscosity_lbf_s_per_ft2 = convert_viscosity_to_lbf_s_per_ft2(viscosity_lbm_per_s_per_ft)
    # print(f"Dynamic Viscosity: {viscosity_lbf_s_per_ft2} lbf-s/ft²")


class EC_Humid_Air(object):
    
    def __init__(self,  **kwargs):

        self.si_propD = {} # key:preferred name, value: value in SI units
        self.eng_propD = {} # key:preferred name, value: value in Engineering units
        self.setProps( **kwargs )

    def setProps(self, **kwargs):
        """
        Calculates various properties of humid air using CoolProp's HAPropsSI function.

        Args:
        - kwargs (dict): Dictionary with three keys corresponding to the input properties and their values.

        Note the values in kwargs are in Engineering units.

        Results in object properties corresponding to state properties in engineering units.
        """
        # Allow user to call setProps with none or partial parameters
        if len(kwargs) == 0:
            kwargs = {'T':536.4 ,'P':PSIA_PER_ATM, 'RelHum':0.5}
        
        # Allow any temperature to be input in degF
        self.eng_inputD = {} # key:CoolProp input name: value=value in Engineering units
        for k,v in kwargs.items():
            if k.endswith('degF'):
                self.eng_inputD[ k[:-4] ] = v + DEG_F_R_OFFSET
            else:
                self.eng_inputD[k] = v

        if len(self.eng_inputD) == 2 and 'P' not in kwargs:
            self.eng_inputD['P'] = PSIA_PER_ATM

        # Ensure the kwargs dictionary has exactly three items
        if len(self.eng_inputD) != 3:
            raise ValueError("Exactly three input properties must be provided")
        
        if 'P' not in self.eng_inputD:
            raise ValueError('"P" must be specified')

        self.si_inputD = {}
        for k,v in self.eng_inputD.items():
            self.si_inputD[k] = toSI_callD[k](v)

        # print( 'eng_inputD =', self.eng_inputD)
        # print( 'si_inputD =', self.si_inputD)

        # Extract keys and values from the self.si_inputD dictionary
        input_keys = list(self.si_inputD.keys())
        input_vals = list(self.si_inputD.values())

        # Iterate through the list of properties and calculate each one
        self.success = True
        for prop in preferred_list:
            try:
                # Place SI results into self.si_propD
                self.si_propD[prop] = CP.HAPropsSI(prop, input_keys[0], input_vals[0], input_keys[1], input_vals[1], input_keys[2], input_vals[2])
            except:
                self.success = False

                print( 'Failed to calc:', prop)
                self.si_propD[prop] = float('inf')

                tb_str = traceback.format_exc()
                if 'ValueError' in tb_str:
                    print( 'ValueError::', tb_str.split('ValueError')[-1])
                else:
                    print( tb_str )

                break # all props will be set to float('inf')
                
        # if not success, set everything to infinity
        if not self.success:
            for prop in preferred_list:
                self.si_propD[prop] = float('inf')


        # include shorter, non-CoolProp name for Conductivity
        self.si_propD['Cond'] = self.si_propD['Conductivity']
                

        # convert properties from SI to Engineering
        for k,v in self.si_propD.items():
            self.eng_propD[k] = toEng_callD[k](v)

        # include shorter, non-CoolProp name for Conductivity
        self.eng_propD['Cond'] = self.eng_propD['Conductivity']

        # assign Engineering properties to self
        for name, value in self.eng_propD.items():
            setattr(self, name, value)
            for syn in param_synonymD[name]:
                if syn != name:
                    setattr(self, syn, value)


        # if 'Y' in self.eng_propD and self.eng_propD['Y'] < 1.0:
        #     # Constants
        #     M_dry_air = 28.96  # Molecular weight of dry air in g/mol
        #     M_water_vapor = 18.0153  # Molecular weight of water vapor in g/mol
        #     Y = self.eng_propD['Y']

        #     MolWt = (1 - Y) * M_dry_air + Y * M_water_vapor
        #     self.eng_propD['MolWt'] = MolWt
        #     self.si_propD['MolWt'] = MolWt

    def get_eng_fmt_size(self):
        """Make a g format sized for largest number of property in propL"""
        # sL = ['%g'%self.eng_propD[name] for name in propL]
        max_len = max([len('%g'%self.eng_propD[name]) for name in preferred_list])
        return '%' + '%ig'%max_len, max_len

    def get_si_fmt_size(self):
        """Make a g format sized for largest number of property in propL"""
        # sL = ['%g'%self.eng_propD[name] for name in propL]
        max_len = max([len('%g'%self.si_propD[name]) for name in preferred_list])
        return '%' + '%ig'%max_len, max_len

    def printSIProps(self):
        """Print a multiline property summary with SI units."""
        self.printProps( eng_units=False )

    def printProps(self, eng_units=True):
        '''
        Print a multiline property summary with Engineering units.
        
        If eng_units=False, SI units will be printed.
        '''

        
        def dict_to_string(D):
            return ', '.join([f"{key}={value:g}" for key, value in D.items()])

        if eng_units:
            s = dict_to_string( self.eng_inputD )
            fmt, max_len = self.get_eng_fmt_size( )
            propD = self.eng_propD.copy()
            unitsD = param_eng_unitsD
            propD['Visc'] *= 1.0E5
        else:
            s = dict_to_string( self.si_inputD )
            fmt, max_len = self.get_si_fmt_size( )
            propD = self.si_propD.copy()
            unitsD = param_si_unitsD
        
        banner(  '-'*4 + ' State Point for Humid Air (%s) '%s +'-'*4, leftMargin=3 )

        for name in print_orderL:
            if name:
                if unitsD[name] == 'degR':
                    alt_units = '%.1f'%(propD[name] - DEG_F_R_OFFSET,)
                    alt_units = '(' + alt_units.strip() + ' degF)'
                elif unitsD[name] == 'psia':
                    alt_units = fmt%(propD[name]/PSIA_PER_ATM,)
                    alt_units = '(' + alt_units.strip() + ' atm)'
                else:
                    alt_units = ''
                print("%10s ="%name, fmt%propD[name], unitsD[name],
                      " :: %s"%param_descD[name], alt_units)
                
                # if eng_units and name=='Visc':
                #     print( '           =', 
                #           fmt%convert_viscosity_to_lbf_s_per_ft2(self.eng_propD['Visc']),
                #            'lbf-s/ft^2', " :: %s"%param_descD[name] )
            else:
                print()


    def print_input_params(self):
        """Print all the legal inputs for Humid Air"""
        banner(  '-'*22 + ' Humid Air Input Parameters ' +'-'*22, leftMargin=3 )
        for name in print_orderL:
            if name:
                if name == 'Cond':
                    desc = preferred_nameD['Conductivity']
                else:
                    desc = preferred_nameD[name]

                # for name, desc in preferred_nameD.items():
                if name in input_set:
                    syn_set = param_synonymD[name]-{name}
                    if syn_set:
                        print( '%10s'%name, '%23s'%param_eng_unitsD[name], '%-34s'%desc, '::AKA', syn_set)
                    else:
                        print( '%10s'%name, '%23s'%param_eng_unitsD[name], '%-34s'%desc)

    def print_output_params(self):
        """Print all the legal outputs for Humid Air"""
        banner(  '-'*22 + ' Humid Air Output Parameters ' +'-'*22, leftMargin=3 )
        for name in print_orderL:
            if name:
                if name == 'Cond':
                    desc = preferred_nameD['Conductivity']
                else:
                    desc = preferred_nameD[name]
            
                syn_set = param_synonymD[name]-{name}
                # if name=='Conductivity': 
                #     name='Cond'
                if syn_set:
                    print( '%10s'%name, '%23s'%param_eng_unitsD[name], '%-34s'%desc, '::AKA', syn_set)
                else:
                    print( '%10s'%name, '%23s'%param_eng_unitsD[name], '%-34s'%desc)

def dev_tests():
    ha = EC_Humid_Air( T=536.4 , RelHum=0.5 )

    # kL = sorted( ha.eng_propD.keys(), key=str.lower)
    # for k in kL:
    #     print( '%10s = %12g'%(k,ha.eng_propD[k]), '= %12G'%ha.si_propD[k])

    print()

    ha.printProps()
    # print('.'*66)
    # ha.printProps(eng_units=False)
    print( 'ha.Cond =', ha.Cond)
    print( 'ha.Visc =', ha.Visc)

    ha.print_input_params()
    ha.print_output_params()

    print( '-'*66 )
    HA = ha = EC_Humid_Air( TdegF=70, RelHum=0.5 )
    HA.printProps()

    print( '-'*66 )
    HA.printSIProps()

if __name__ == "__main__":

    dev_tests()



import matplotlib.pyplot as plt
import numpy as np
from engcoolprop.ec_humid_air import EC_Humid_Air, DEG_F_R_OFFSET

HA = EC_Humid_Air()

relhumArr = np.linspace(.05, 1, 20)
drybulbL = [100, 90, 80, 70] # degF

for drybulb in drybulbL:

    dewptL = []
    wetbulbL = []
    for RelHum in relhumArr:
        HA.setProps( TdegF=drybulb, RelHum=RelHum)
        
        dewptL.append( HA.Tdp - DEG_F_R_OFFSET )
        wetbulbL.append( HA.Twb - DEG_F_R_OFFSET )

    plt.plot( dewptL, wetbulbL, label='Dry Bulb=%g F'%drybulb)
plt.grid( True )
plt.title( 'Humid Air Temperatures')
plt.xlabel( 'Dew Point (degF)')
plt.ylabel( 'Wet Bulb Temperature (degF)')
plt.legend( loc='best' )

plt.savefig( 'dew_point.png', dpi=200)
plt.show()



import matplotlib.pyplot as plt
import numpy as np
from engcoolprop.ec_incomp_soln import EC_Incomp_Soln

P = 1000
for pcent in [60, 40, 20, 0]:

    # Create incompressible solution object. (without specifying state point)
    ec_soln = EC_Incomp_Soln(symbol="MEG-%i%%"%pcent, auto_fix_value_errors=True, show_warnings=0)

    tArr = np.linspace(ec_soln.Tmin, ec_soln.Tmax, 50)

    densL = []
    for T in tArr:
        ec_soln.setTP( T, P)
        densL.append( ec_soln.D )

    plt.plot( tArr, densL, label="MEG-%i%%"%pcent)

plt.grid( True )
plt.title( 'MEG Solution Densities')
plt.xlabel( 'Temperature (degR)')
plt.ylabel( 'Density (lbm/cuft)')
plt.legend( loc='best' )

plt.savefig( 'MEG_pcent_D.png', dpi=200)
plt.show()



from engcoolprop.ec_humid_air import EC_Humid_Air
from engcoolprop.banner import banner
        
def dict_to_string(D):
    return ', '.join([f"{key}={value:g}" for key, value in D.items()])

def make_ha_obj( init_str ):
    HA = eval( init_str )
    print( '%36s'%init_str, '#', dict_to_string(HA.eng_inputD), end='\n\n' )

banner(  '------ Results of different calls to EC_Humid_Air ------', leftMargin=10)

margin = ' ' * 16
print( margin, 'Can initialize with no input parameters')
make_ha_obj( 'EC_Humid_Air()' )

print( margin, 'Can omit pressure (P) and 1 atm will be assumed')
make_ha_obj( 'EC_Humid_Air(T=600, R=1)' )

print( margin, 'Can append "degF" to any temperature and degR will result')
make_ha_obj( 'EC_Humid_Air(TdegF=70, RelHum=.6)' )

print( margin, 'Can use all synonyms for inputs (e.g. T_db, Tdb, T)')
make_ha_obj( 'EC_Humid_Air(P=14.7, RH=.3, Tdb=530)' )





import matplotlib.pyplot as plt
from engcoolprop.ec_humid_air import EC_Humid_Air

HA = EC_Humid_Air()

for RelHum in [1.0, 0.7, 0.5, 0.3, 0.1]:

    tL = [500 + i for i in range(61)]
    hL = []
    for T in tL:
        HA.setProps( Tdb=T, RelHum=RelHum)
        hL.append( HA.P_w )

    plt.plot( tL, hL, label='RelHum=%g'%RelHum)
plt.grid( True )
plt.title( 'Humid Air Water Partial Pressure')
plt.xlabel( 'Dry Bulb Temperature (degR)')
plt.ylabel( 'Water Partial Pressure (psia)')
plt.legend( loc='best' )

plt.savefig( 'water_partial_pressure.png', dpi=200)
plt.show()
